!= $BK0OB?e>x5$05$K1~$8$?I=LL5$05$NJQ99(B
!
!= modification of surface pressure according to saturated vapor pressure
!
! Authors::   Masaki Ishiwatari
! Version::   $Id: moist_conv_adjust.f90,v 1.7 2011-12-02 04:06:36 yot Exp $ 
! Tag Name::  $Name: dcpam5-20120922 $
! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]

module equilib_modify_ps
  !
  != $BCOI=LL29EY$K1~$8$FK0OB?e>x5$J,05$rI=LL5$05$KB-$79~$`(B
  !
  != stratospherice optical depth of equilibrium solution with gray radiation
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! $B3%?'J|<MBPN.J?9U2r$K$*$1$k@.AX7w$N8w3XE*?<$5$r7W;;$9$k(B
  !
  ! In this module, stratospherice optical depth of equilibrium solution 
  ! with gray radiationis obtained.
  !
  !== Procedures List
  ! 
  ! EquilibModifyPs :: $B3%?'J|<MBPN.J?9U2r$K$*$1$k@.AX7w$N8w3XE*?<$5$r7W;;(B
  ! ------------------ :: ------------------------------------------------
  ! EquilibModifyPs :: stratospherice optical depth of equilibrium solution 
  !
  !== NAMELIST
  !
  ! NAMELIST#equilib_modify_ps_nml
  !

  ! $B%b%8%e!<%k0zMQ(B ; USE statements
  !

  ! $B3J;RE@@_Dj(B
  ! Grid points settings
  !
  use gridset, only: imax, & ! $B7PEY3J;RE@?t(B. 
                             ! Number of grid points in longitude
    &                jmax, & ! $B0^EY3J;RE@?t(B. 
                             ! Number of grid points in latitude
    &                kmax    ! $B1tD>AX?t(B. 
                             ! Number of vertical level

  ! $B<oJL7?%Q%i%a%?(B
  ! Kind type parameter
  !
  use dc_types, only: DP, &      ! $BG\@:EY<B?t7?(B. Double precision. 
    &                 STRING     ! $BJ8;zNs(B.       Strings. 

  ! NAMELIST $B%U%!%$%kF~NO$K4X$9$k%f!<%F%#%j%F%#(B
  ! Utilities for NAMELIST file input
  !
  use namelist_util, only: MaxNmlArySize
                              ! NAMELIST $B$+$iFI$_9~$`G[Ns$N:GBg%5%$%:(B. 
                              ! Maximum size of arrays loaded from NAMELIST

  ! $B%a%C%;!<%8=PNO(B
  ! Message output
  !
  use dc_message, only: MessageNotify

  ! $B@k8@J8(B ; Declaration statements
  !
  implicit none
  private

  ! $B8x3+<jB3$-(B
  ! Public procedure
  !
  public:: EquilibModifyPs
  public:: EquilibModifyPsInit

  ! $B8x3+JQ?t(B
  ! Public variables
  !
  logical, save :: equilib_modify_ps_inited = .false.
                              ! $B=i4|@_Dj%U%i%0(B. 
                              ! Initialization flag

  ! $BHs8x3+JQ?t(B
  ! Private variables
  !
  character(*), parameter:: module_name = 'equilib_modify_ps'
                              ! $B%b%8%e!<%k$NL>>N(B. 
                              ! Module name
  character(*), parameter:: version = &
    & '$Name: dcpam5-20120922 $' // &
    & '$Id: moist_conv_adjust.f90,v 1.7 2011-12-02 04:06:36 yot Exp $'
                              ! $B%b%8%e!<%k$N%P!<%8%g%s(B
                              ! Module version
contains

  !--------------------------------------------------------------------------------------

    subroutine EquilibModifyPs( &
      & xy_SurfTemp, &
      & xy_Ps &
      & )
    !
    ! 
    ! $BJ?9U2r$N@.AX7w8w3XE*8|$5$r7W;;$9$k(B.
    !
    ! stratospheric optical depth of equilibrium solution
    !

    ! $B%b%8%e!<%k0zMQ(B ; USE statements
    !

    ! $BJ*M}Dj?t@_Dj(B
    ! Physical constants settings
    !
    use constants, only: &
      & MolWtDry, &
                              ! $ M $ [kg mol-1]. 
                              ! $B4%AgBg5$$NJ?6QJ,;RNL(B. 
                              ! Mean molecular weight of dry air
      & MolWtWet
                              ! $ M_v $ [kg mol-1]. 
                              ! $B6E7k@.J,$NJ?6QJ,;RNL(B. 
                              ! Mean molecular weight of condensible elements

    ! $B;~9o4IM}(B
    ! Time control
    !
    use timeset, only: &
      & DelTime, &            ! $ \Delta t $
      & TimeN, &              ! $B%9%F%C%W(B $ t $ $B$N;~9o(B. Time of step $ t $. 
      & TimesetClockStart, TimesetClockStop

    ! $B%R%9%H%j%G!<%?=PNO(B
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoPut

    ! $BK0OBHf<>$N;;=P(B
    ! Evaluate saturation specific humidity
    !
    use saturate, only:                             &
      & xy_CalcQVapSat

    ! $B@k8@J8(B ; Declaration statements
    !
    implicit none

    real(DP):: xy_Ps       ( 0:imax-1, 1:jmax ) !" $BI=LL5$05(B
    real(DP):: xy_SurfTemp ( 0:imax-1, 1:jmax )  !" $BHf<>(B  $B#q(B

    integer:: I,J,K

    xy_Ps = xy_Ps  &
     & *( 1.0d0 + xy_CalcQVapSat(xy_SurfTemp,xy_Ps)/ (MolWtWet / MolWtDry) )

  end subroutine EquilibModifyPs

  !--------------------------------------------------------------------------------------

  subroutine EquilibModifyPsInit
    !
    ! equilib_tropopause $B%b%8%e!<%k$N=i4|2=$r9T$&(B.
    ! NAMELIST#equilib_tropopause_nml $B$NFI$_9~$_$O$3$N<jB3$-$G9T$o$l$^$9(B. 
    !
    ! "equilib_tropopause" module is initialized. 
    ! "NAMELIST#equilib_tropopause_nml" is loaded in this procedure. 
    !

    ! $B%b%8%e!<%k0zMQ(B ; USE statements
    !

    ! NAMELIST $B%U%!%$%kF~NO$K4X$9$k%f!<%F%#%j%F%#(B
    ! Utilities for NAMELIST file input
    !
    use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid

    ! $B%U%!%$%kF~=PNOJd=u(B
    ! File I/O support
    !
    use dc_iounit, only: FileOpen

    ! $B<oJL7?%Q%i%a%?(B
    ! Kind type parameter
    !
    use dc_types, only: STDOUT ! $BI8=`=PNO$NAuCVHV9f(B. Unit number of standard output

    ! $BJ8;zNsA`:n(B
    ! Character handling
    !
    use dc_string, only: StoA

    ! $B%R%9%H%j%G!<%?=PNO(B
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoAddVariable

    ! $B@k8@J8(B ; Declaration statements
    !
    implicit none

    integer:: unit_nml        ! NAMELIST $B%U%!%$%k%*!<%W%sMQAuCVHV9f(B. 
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST $BFI$_9~$_;~$N(B IOSTAT. 
                              ! IOSTAT of NAMELIST read

    ! NAMELIST $BJQ?t72(B
    ! NAMELIST group name
    !
!    namelist /equilib_tropopause_nml/ &
!      & RH, ItrtMax

          ! $B%G%U%)%k%HCM$K$D$$$F$O=i4|2=<jB3(B "moist_conv_adjust#CumAdjInit" 
          ! $B$N%=!<%9%3!<%I$r;2>H$N$3$H(B. 
          !
          ! Refer to source codes in the initialization procedure
          ! "moist_conv_adjust#MoistConvAdjustInit" for the default values. 
          !

    ! $B<B9TJ8(B ; Executable statement
    !

    if ( equilib_modify_ps_inited ) return

    ! $B%G%U%)%k%HCM$N@_Dj(B
    ! Default values settings
!    XXXXXXXX  = 1.0d0

    !
!!$    FlagUse = .true.

    ! NAMELIST $B$NFI$_9~$_(B
    ! NAMELIST is input
    !
!    if ( trim(namelist_filename) /= '' ) then
!      call FileOpen( unit_nml, &          ! (out)
!        & namelist_filename, mode = 'r' ) ! (in)
!
!      rewind( unit_nml )
!      read( unit_nml,                  &  ! (in)
!        & nml = moist_adiabat_nml, &  ! (out)
!        & iostat = iostat_nml )           ! (out)
!      close( unit_nml )
!
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      if ( iostat_nml == 0 ) write( STDOUT, nml = moist_adiabat_nml )
!    end if

    ! $B%$%F%l!<%7%g%s2s?t(B, $BIT0BDj$N5vMF8m:9$N%A%'%C%/(B
    ! Check number of iteration, admissible error of unstability
    !
!    call NmlutilAryValid( module_name, & ! (in)
!      & AdjustCriterion, 'AdjustCriterion', &      ! (in)
!      & ItrtMax,    'ItrtMax' )          ! (in)


    ! $B%R%9%H%j%G!<%?=PNO$N$?$a$N$X$NJQ?tEPO?(B
    ! Register of variables for history data output
    !
!    call HistoryAutoAddVariable( 'RainCumulus', &
!      & (/ 'lon ', 'lat ', 'time' /), &
!      & 'precipitation by cumulus scheme', 'W m-2' )
!    call HistoryAutoAddVariable( 'DTempDtCumulus', &
!      & (/ 'lon ', 'lat ', 'sig ', 'time' /), &
!      & 'cumulus condensation heating', 'K s-1' )
!    call HistoryAutoAddVariable( 'DQVapDtCumulus', &
!      & (/ 'lon ', 'lat ', 'sig ', 'time' /), &
!      & 'cumulus condensation moistening', 'kg kg-1 s-1' )

    ! $B0u;z(B ; Print
    !
    call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )
!    call MessageNotify( 'M', module_name, '  ItrtMax              = %d', i = (/ ItrtMax /) )
!    call MessageNotify( 'M', module_name, '  AdjustCriterion      = (/ %*r /)', &
!      & r = real( AdjustCriterion(1:ItrtMax) ), n = (/ ItrtMax /) )
!    call MessageNotify( 'M', module_name, '' )
!    call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )

    equilib_modify_ps_inited = .true.

  end subroutine EquilibModifyPsInit

end module equilib_modify_ps
